//
//  ECellBluetoothManager.m
//  ECellBluetooth_Example
//
//  Created by lemo. on 2019/5/14.
//  Copyright © 2019 chentingfeng. All rights reserved.
//

#import "ECellBluetoothManager.h"
#import <ECellBluetooth/ECellBLEEnum.h>
#import <ECellBluetooth/ECellBLEItem.h>
#import <ECellBluetooth/ECellDeviceInfoModel.h>
#import <ECellBluetooth/ECellBLEDataParse.h>
#import <ECellBluetooth/ECellCommandProvider.h>
#import <ECellBluetooth/ECellBLENotify.h>

@interface ECellBluetoothManager () <CBPeripheralDelegate,CBCentralManagerDelegate>

//* 是否需要重连机制 默认 YES
@property (nonatomic,assign) BOOL isReconnect;
/** 扫描到的设备  */
@property (nonatomic, strong) NSMutableArray <ECellPeripheralModel *>*scanDevices;
/** 指令写入通道 */
@property (nonatomic, strong) CBCharacteristic *writeCharacteristic;
/** UI表盘写入通道 */
@property (nonatomic, strong) CBCharacteristic *dialWriteCharacteristic;
/** 蓝牙指令队列 */
@property (nonatomic, strong) NSMutableArray <ECellBLEItem *>*commandBleItems;
/** 是否正在执行 */
@property (nonatomic, assign) BOOL executing;
/** 超时定时器 */
@property (nonatomic, strong) NSTimer *checkoutTimer;
/** 超时计数 */
@property (nonatomic, assign) NSInteger timeNumber;
/** 当前执行的命令 */
@property (nonatomic, strong) ECellBLEItem *currenItem;
/** 当前接收的数据 */
@property (nonatomic, strong) NSMutableData *currenData;
/** 总数据包 */
@property (nonatomic, strong) NSMutableData *totalData;

/** 重连外设 */
@property (nonatomic,strong) CBPeripheral *reconnectPeripheral;
// 外部只读属性内部重新声明
@property (nonatomic,assign,readwrite) BOOL poweredOn;
@property (nonatomic,assign,readwrite) BOOL connected;

@end

@implementation ECellBluetoothManager

/**
 * @brief 初始化蓝牙管理中心对象
 * Initialize the Bluetooth Management Center object
 * @return ECellBluetoothManager
 */
+ (__kindof ECellBluetoothManager *)shareInstance {
    static ECellBluetoothManager *share = nil;
    static dispatch_once_t oneToken;
    dispatch_once(&oneToken, ^{
        share = [[ECellBluetoothManager alloc] init];
    });
    return share;
}

- (instancetype)init {
    self = [super init];
    if (self) {
        CBCentralManager *centralManager = [[CBCentralManager alloc] initWithDelegate:self queue:nil];
        self.centralManager = centralManager;
        self.isReconnect = YES;
        self.currenData = [[NSMutableData alloc] init];
        self.totalData = [[NSMutableData alloc] init];
        self.commandTimeOutNumber = 8;
        self.rssiNum = 80;
        self.isJL = NO;
    }
    return self;
}


#pragma mark - 外部方法

/**
 注册SDK服务 | Register for SDK service
 
 @param outputSdkLog 是否输出蓝牙协议运行日志 | Whether to output the Bluetooth protocol running log
 @param isReconnect 是否需要重连机制 | Whether a reconnect mechanism is required
 */
- (void)registrationServices:(BOOL)outputSdkLog
                 IsReconnect:(BOOL)isReconnect {
    ECellBluetoothReportLoggerIsOpen = outputSdkLog;
    self.isReconnect = isReconnect;
}

/**
 开始扫描 | Start scanning
 */
- (void)startScan {
    [self stopScan];
    // 清空上一次扫描设备列表
    [self.scanDevices removeAllObjects];
    // 添加系统已配对设备
    [self addPairedConnectedArray];
    // 开启扫描过滤
    [self.centralManager scanForPeripheralsWithServices:@[[CBUUID UUIDWithString:kScanServiceUUID]] options:nil];
}

/**
 停止扫描 | Stop scanning
 */
- (void)stopScan {
    if (self.centralManager.isScanning) {
        [self.centralManager stopScan];
    }
}

/**
 * @brief 选择外围设备连接 | Select peripheral connection
 * @param model ECellPeripheralModel 对象 | ECellPeripheralModel object
 */
- (void)connectDeviceWithModel:(ECellPeripheralModel *)model {
    [self stopScan];
    [self.centralManager connectPeripheral:model.peripheral options:nil];
}

/**
 断开当前外围设备的连接 (将会清空设备信息不再重连) | Disconnect the current peripheral device (The device information will be cleared and will not be reconnected)
 */
- (void)cancelCurrentPeripheralConnection {
    // 手动断开设备连接，清空本地设备信息
    [[NSUserDefaults standardUserDefaults] removeObjectForKey:ECellLocalDeviceInfo];
    // 未连接不执行断开(停止重连动作)
    if (!self.currenPeripheral) {
        [self startAndStopReconnect:NO];
        return;
    }
    [self.centralManager cancelPeripheralConnection:self.currenPeripheral];
}

/**
 监听设备主动上报数据
 */
- (void)listenResult:(ResultReportBlock)result {
    self.resultReportBlock = result;
}

#pragma mark - 内部方法

/**
 写入数据(指令通道)
 */
- (void)writeWithBleItem:(ECellBLEItem *)item {
    if (!self.connected) {
        if (item.result) {
            item.result(ECell_BLE_COMMAND_NOT_CONNECTED_TYPE, nil);
        }
        return;
    }
    [self.commandBleItems addObject:item];
    [self loopRequest];
}

/**
 循环串行 执行蓝牙请求(先进先出)
 */
- (void)loopRequest {
    if (self.commandBleItems.count > 0 && self.executing == NO) {
        self.executing = true;
        // 超时定时器开启
        self.timeNumber = 0;
        NSTimer *checkoutTimer = [NSTimer scheduledTimerWithTimeInterval:1.0 target:self selector:@selector(clickTimeout:) userInfo:nil repeats:YES];
        self.checkoutTimer = checkoutTimer;
        // 分包发送数据
        ECellBLEItem *bleItem = self.commandBleItems.firstObject;
        self.currenItem = bleItem;
        [self handleWithByteBuffer:bleItem.byteBuffer];
        ECellLog(@"[BLUETOOTH DATA] SEND DATA %@",bleItem.byteBuffer);
    }
}

/**
 分包数据发送处理
 */
- (void)handleWithByteBuffer:(NSMutableData *)byteBuffer {
    // 容错处理(读不到写入特征值断线重连)
    if (!self.writeCharacteristic) {
        [self.centralManager cancelPeripheralConnection:self.currenPeripheral];
        return;
    }
    // 大于20字节则分包发送
    if (byteBuffer.length > 20) {
        NSInteger count = byteBuffer.length / 20;
        for (int i = 0; i <= count; i++) {
            NSData *data = [[NSData alloc] init];
            if (i < count) {
                data = [byteBuffer subdataWithRange:NSMakeRange(i * 20, 20)];
            }else if (byteBuffer.length % 20) {
                // 最后不足20字节的数据
                data = [byteBuffer subdataWithRange:NSMakeRange(i * 20, byteBuffer.length % 20)];
            }
            if (data) {
                // 写入数据
                [self.currenPeripheral writeValue:data forCharacteristic:self.writeCharacteristic type:CBCharacteristicWriteWithoutResponse];
            }
        }
    }else {
        [self.currenPeripheral writeValue:byteBuffer forCharacteristic:self.writeCharacteristic type:CBCharacteristicWriteWithoutResponse];
    }
}

/**
 超时处理
 */
- (void)clickTimeout:(NSTimer *)timer {
    self.timeNumber++;
    if (self.timeNumber >= self.commandTimeOutNumber) {
        // 超时回调
        if (self.currenItem.result) {
            self.currenItem.result(ECell_BLE_COMMAND_TIME_OUT_TYPE, nil);
        }
        [self completeAndTimeouthandle];
    }
}

/**
 任务完成 || 超时 处理
 */
- (void)completeAndTimeouthandle {
    self.currenData = [NSMutableData new];
    self.totalData = [NSMutableData new];
    self.executing = false;
    self.timeNumber = 0;
    [self.checkoutTimer invalidate];
    self.checkoutTimer = nil;
    [self.commandBleItems removeObject:self.currenItem];
    [self loopRequest];
}


/**
 处理接受到收到的数据
 
 @param receiveData 接收数据
 */
- (void)handleReceiveData:(NSData *)receiveData ParseResult:(ParseCmdResultBlock)resultBlock {
  //  ECellLog(@"[BLUETOOTH DATA] RECEIVE DATA %@",receiveData);
    // 存在队列的情况下，重置超时计时
    if (self.commandBleItems.count) {
        self.timeNumber = 0;
    }
    // 处理命令后的返回数据
    [self.currenData appendData:receiveData];
    // 判断数据是否完整
    Byte *currentDataBytes = (Byte *)[self.currenData bytes];
    int contenLenght = (currentDataBytes[2] & 0xff) << 8 | (currentDataBytes[3] & 0xff);
    int actualLenght = (int)self.currenData.length - 8;
    // 接收到的数据包完整
    if (contenLenght == actualLenght) {
        // 判断序号是否存在当前队列
        int sid = (currentDataBytes[6] & 0xff) << 8 | (currentDataBytes[7] & 0xff);
        ECellBLEItem *item = [self requestItemFromIndex:sid];
        if (item) {
            ECellLog(@"[BLUETOOTH DATA] RECEIVE REPLY DATA %@",receiveData);
            // 判断err,1:成功 0:失败
            BOOL err = [self checkDataWithData:self.currenData ByteIndex:1 bitIndex:2];
            if (err) {
                // 大于200bytes字节的数据不做CRC处理
                BOOL isCrc = YES;
                if (contenLenght <= 200) {
                    // CRC校验(失败则返回重发)
                    if (![self checkCRCWithData:self.currenData]) {
                        isCrc = NO;
                    }
                }
                // 判断是否有后续数据包返回 1:有 0:无
                BOOL isData = [self checkDataWithData:self.currenData ByteIndex:1 bitIndex:1];
                if (!isData && !isCrc) {
                    // 不存在后续数据并且数据检验失败直接重发
                    if (resultBlock) {
                        resultBlock(ECell_BLE_COMMAND_PARSE_FAIL_TYPE,nil);
                    }
                    return;
                }
                if (isData) {
                    // 还有后续数据则将当前完整数据包以分隔符拼接
                    [self.totalData appendData:self.currenData];
                    self.currenData = [NSMutableData new];
                    [self.totalData appendData:[DataSeparator dataUsingEncoding:NSUTF8StringEncoding]];
                    return;
                }else{
                    // 拼接最后完整数据包
                    [self.totalData appendData:self.currenData];
                    // 解析完整数据
                    NSDictionary *dict = [ECellBLEDataParse parse:self.totalData cmd:self.currenItem.cmdKey];
                    // 回调数据
                    if (resultBlock) {
                        resultBlock(ECell_BLE_COMMAND_PARSE_SUCCESS_TYPE,dict);
                    }
                    [self completeAndTimeouthandle];
                }
            }
        }else {
            // 不存在此队列(设备主动发送过来的数据)
            [self handleDeviceControlReceiveData:self.currenData];
            self.currenData = [NSMutableData new];
        }
    }else {
        // 数据不完整返回拼接
        if (resultBlock) {
            resultBlock(ECell_BLE_COMMAND_PARSE_NEXT_TYPE,nil);
        }
        return;
    }
}

/**
 处理接受到收到的数据
 
 @param receiveData 接收数据
 */
- (void)handleReceiveData:(NSData *)receiveData {
    //  ECellLog(@"[BLUETOOTH DATA] RECEIVE DATA1 %@",receiveData);
    // 存在队列的情况下，重置超时计时
    if (self.commandBleItems.count) {
        self.timeNumber = 0;
    }
    // 处理命令后的返回数据
    [self.currenData appendData:receiveData];
    // 判断数据是否完整
    Byte *currentDataBytes = (Byte *)[self.currenData bytes];
    int contenLenght = (currentDataBytes[2] & 0xff) << 8 | (currentDataBytes[3] & 0xff);
    int actualLenght = (int)self.currenData.length - 8;
    // 接收到的数据包完整
    if (contenLenght == actualLenght) {
        // 判断序号是否存在当前队列
        int sid = (currentDataBytes[6] & 0xff) << 8 | (currentDataBytes[7] & 0xff);
        ECellBLEItem *item = [self requestItemFromIndex:sid];
        if (item) {
            ECellLog(@"[BLUETOOTH DATA] RECEIVE REPLY DATA1 %@",receiveData);
            // 判断err,1:成功 0:失败
            BOOL err = [self checkDataWithData:self.currenData ByteIndex:1 bitIndex:2];
            if (err) {
                // 大于200bytes字节的数据不做CRC处理
                BOOL isCrc = YES;
                if (contenLenght <= 200) {
                    // CRC校验(失败则返回重发)
                    if (![self checkCRCWithData:self.currenData]) {
                        isCrc = NO;
                    }
                }
                // 判断是否有后续数据包返回 1:有 0:无
                BOOL isData = [self checkDataWithData:self.currenData ByteIndex:1 bitIndex:1];
                if (!isData && !isCrc) {
                    // 不存在后续数据并且数据检验失败直接重发
                    return;
                }
                if (isData) {
                    // 还有后续数据则将当前完整数据包以分隔符拼接
                    [self.totalData appendData:self.currenData];
                    self.currenData = [NSMutableData new];
                    [self.totalData appendData:[DataSeparator dataUsingEncoding:NSUTF8StringEncoding]];
                    return;
                }else{
                    // 拼接最后完整数据包
                    [self.totalData appendData:self.currenData];
                    // 解析完整数据
                    NSDictionary *dict = [ECellBLEDataParse parse:self.totalData cmd:self.currenItem.cmdKey];
                    // 回调数据
                    if (self.currenItem.result) {
                        self.currenItem.result(ECell_BLE_COMMAND_SUCCESS_TYPE, dict);
                    }
                    [self completeAndTimeouthandle];
                }
            }
        }else {
            // 不存在此队列(设备主动发送过来的数据)
            //ECellLog(@"不存在此队列(设备主动发送过来的数据)");
            [self handleDeviceControlReceiveData:self.currenData];
            self.currenData = [NSMutableData new];
        }
    }else {
        // 数据不完整返回拼接
        return;
    }
}

/**
 处理设备发送过来的指令(找手机,拍照功能，实时步数，实时心率)
 */
- (void)handleDeviceControlReceiveData:(NSMutableData *)receiveData {
    ECellLog(@"[BLUETOOTH DATA] RECEIVE REPORT DATA %@",receiveData);
    // 检查是否为指令
    if (((Byte*)([receiveData bytes]))[0] != 0xCB) {
        return;
    }
    // 判断err是否正确
    BOOL err = [self checkDataWithData:receiveData ByteIndex:1 bitIndex:2];
    if (err) {
        ECell_BLECMD_KEY cmdKey = [[NSString stringWithFormat:@"%02d", ((Byte*)([receiveData bytes]))[10]] integerValue];
        NSDictionary *dict = [ECellBLEDataParse parse:receiveData cmd:cmdKey];
        ECellLog(@"-->%@",dict);
        ECell_BLE_REPORT_COMMAND reportCmd = ECell_BLE_REPORT_COMMAND_realStep;
        switch (cmdKey) {
            case ECell_BLECMD_KEY_realStep:
                reportCmd = ECell_BLE_REPORT_COMMAND_realStep;
                break;
                
            case ECell_BLECMD_KEY_realHeartRate:
                reportCmd = ECell_BLE_REPORT_COMMAND_realHeartRate;
                break;
                
            case ECell_BLECMD_KEY_realBloodPressure:
                reportCmd = ECell_BLE_REPORT_COMMAND_realBloodPressure;
                break;
                
            case ECell_BLECMD_KEY_realBloodOxygen:
                reportCmd = ECell_BLE_REPORT_COMMAND_realBloodOxyge;
                break;
                
            case ECell_BLECMD_KEY_camera:
                reportCmd = ECell_BLE_REPORT_COMMAND_cameraControl;
                break;
                
            case ECell_BLECMD_KEY_findPhone:
                reportCmd = ECell_BLE_REPORT_COMMAND_findPhone;
                break;
                
            case ECell_BLECMD_KEY_realTimesync:
            case ECell_BLECMD_KEY_syncHealth:
                reportCmd = ECell_BLE_REPORT_COMMAND_realSport;
                break;
            default:
//                // 回复设备发出通知
//                [self checkCRCHandleWithData:receiveData confirm:YES];
//                return;
                break;
        }
        if (IsDelegate(self.delegate, getDeviceReportData:json:)) {
            [self.delegate getDeviceReportData:reportCmd json:dict];
        }
        
        if (self.resultReportBlock) {
            self.resultReportBlock(reportCmd, dict);
        }
        
        // 回复设备发出通知
        [self checkCRCHandleWithData:receiveData confirm:YES];
        return;
    }
}

/**
 设备断开连接处理
 */
- (void)handleDeviceDisConnect {
    // 状态修改
    self.connected = NO;
    self.currenPeripheral = nil;
    // 移除断开设备的请求队列
    [self.commandBleItems removeAllObjects];
    // 完成当前请求
    if (self.currenItem) {
        [self completeAndTimeouthandle];
    }
    [self startAndStopReconnect:YES];
}

/**
 重连系统已配对设备
 */
- (void)startAndStopReconnect
{
    [self startAndStopReconnect:YES];
}

/**
 蓝牙设备重连

 @param isStart 开启停止
 */
- (void)startAndStopReconnect:(BOOL)isStart {
    if (!isStart || !self.isReconnect || !self.poweredOn) {
        // 取消重连
        if (self.reconnectPeripheral) {
            [self.centralManager cancelPeripheralConnection:self.reconnectPeripheral];
        }
        return;
    }
    // 获取本地设备
    ECellDeviceInfoModel *deviceModel = [ECellDeviceInfoModel currentModel];
    
    NSLog(@"-->startAndStopReconnect");
    if (!deviceModel.uuidStr) {
        return;
    }
    // 重连系统已配对设备
    NSUUID *uuid = [[NSUUID alloc] initWithUUIDString:deviceModel.uuidStr];
    NSArray *systemPeripherals = [self.centralManager retrievePeripheralsWithIdentifiers:@[uuid]];
    if (systemPeripherals.count) {
        CBPeripheral *peripheral = systemPeripherals.firstObject;
        self.reconnectPeripheral = peripheral;
        [self.centralManager connectPeripheral:self.reconnectPeripheral options:nil];
    }
}

#pragma mark - CBPeripheralDelegate,CBCentralManagerDelegate

// 系统蓝牙状态监听
- (void)centralManagerDidUpdateState:(CBCentralManager *)central {
    NSInteger state = ECell_BLE_MANAGER_STATE_POWEREDOFF;
    self.poweredOn = NO;
    if (central.state == CBCentralManagerStatePoweredOn) {
        self.poweredOn = YES;
        state = ECell_BLE_MANAGER_STATE_POWEREDON;
        // 开启重连
        [self startAndStopReconnect:YES];
    }else {
        // 处理断开
        [self handleDeviceDisConnect];
    }
    [kNotification postNotificationName:ECellBLENotificationDidUpdateState object:@(state)];
    if (IsDelegate(self.delegate, bluetoothManager:didUpdateState:)) {
        [self.delegate bluetoothManager:self didUpdateState:state];
    }
}

// 扫描设备
- (void)centralManager:(CBCentralManager *)central didDiscoverPeripheral:(CBPeripheral *)peripheral advertisementData:(NSDictionary *)advertisementData RSSI:(NSNumber *)RSSI {
    // 信号值过滤
    if (abs(RSSI.intValue) > self.rssiNum) {
        return;
    }
    // 扫描设备模型返回
    ECellPeripheralModel *peripheralModel = [[ECellPeripheralModel alloc] init];
    peripheralModel.peripheral = peripheral;
    peripheralModel.name = peripheral.name;
    peripheralModel.uuidStr = peripheral.identifier.UUIDString;
    peripheralModel.rssi = RSSI.integerValue;
    peripheralModel.advertisementData = advertisementData;
    [self.scanDevices addObject:peripheralModel];
    if (IsDelegate(self.delegate, bluetoothManager:allDevices:otaDevices:)) {
        [self.delegate bluetoothManager:self allDevices:self.scanDevices otaDevices:@[]];
    }
}

// 连接成功回调（此处仅处理订阅服务）
- (void)centralManager:(CBCentralManager *)central didConnectPeripheral:(CBPeripheral *)peripheral {
    peripheral.delegate = self;
    [peripheral discoverServices:nil];
}

// 连接外设失败
- (void)centralManager:(CBCentralManager *)central didFailToConnectPeripheral:(CBPeripheral *)peripheral error:(NSError *)error {
    if (IsDelegate(self.delegate, bluetoothManager:connectPeripheralError:)) {
        [self.delegate bluetoothManager:self connectPeripheralError:error];
    }
}

// 已经断开连接
- (void)centralManager:(CBCentralManager *)central didDisconnectPeripheral:(CBPeripheral *)peripheral error:(NSError *)error{
    [self handleDeviceDisConnect];
    [kNotification postNotificationName:ECellBLENotificationAtDisconnect object:peripheral];
    if (IsDelegate(self.delegate, bluetoothManager:centerManager:disConnectPeripheral:isOatMode:)) {
        [self.delegate bluetoothManager:self centerManager:central disConnectPeripheral:peripheral isOatMode:NO];
    }
}

// 发现外设服务
- (void)peripheral:(CBPeripheral *)peripheral didDiscoverServices:(NSError *)error {
    for (CBService *service in peripheral.services) {
        [peripheral discoverCharacteristics:nil forService:service];
    }
    if (IsDelegate(self.delegate, bluetoothPeripheral:didDiscoverServices:))
    {
        [self.delegate bluetoothPeripheral:peripheral didDiscoverServices:error];
    }
}

// 发现外设服务中的特征值
- (void)peripheral:(CBPeripheral *)peripheral didDiscoverCharacteristicsForService:(CBService *)service error:(NSError *)error {
    if ([service.UUID isEqual:[CBUUID UUIDWithString:kServiceUUID]]) {
        for (CBCharacteristic *characteristic in service.characteristics ) {
            if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:kNotifyCharacteristic]]) {
                [peripheral setNotifyValue:true forCharacteristic:characteristic];
            }
            if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:kWriteCharacteristic]]) {
                self.writeCharacteristic = characteristic;
            }
        }
    }
    if (IsDelegate(self.delegate, bluetoothPeripheral:didDiscoverCharacteristicsForService:error:))
    {
        [self.delegate bluetoothPeripheral:peripheral didDiscoverCharacteristicsForService:service error:error];
    }
}

// characteristic订阅状态改变
- (void)peripheral:(CBPeripheral *)peripheral didUpdateNotificationStateForCharacteristic:(CBCharacteristic *)characteristic error:(nullable NSError *)error {
    // 订阅成功处理(此处才作为连接成功的标准)
    if (characteristic.isNotifying && [characteristic.UUID isEqual:[CBUUID UUIDWithString:kNotifyCharacteristic]])
    {
        // 修改状态
        self.connected = YES;
        self.reconnectPeripheral = nil;
        // 获取已连接外设
        self.currenPeripheral = peripheral;
        // 获取固件信息
        [ECellCommandProvider getFirmwareInfoCommand:^(ECell_BLE_COMMAND_ERROR_TYPE errorType, NSDictionary * _Nullable json) {
            NSMutableDictionary *deviceInfo = [[NSMutableDictionary alloc] initWithDictionary:json];
            NSString *uuidStr = self.currenPeripheral.identifier.UUIDString;
            [deviceInfo setObject:uuidStr forKey:@"uuidStr"];
            [[NSUserDefaults standardUserDefaults] setObject:deviceInfo forKey:ECellLocalDeviceInfo];
            // 连接成功
            if (IsDelegate(self.delegate, bluetoothManager:centerManager:didConnectPeripheral:isOatMode:)) {
                [self.delegate bluetoothManager:self centerManager:self.centralManager didConnectPeripheral:peripheral isOatMode:NO];
            }
            [kNotification postNotificationName:ECellBLENotificationConnected object:nil];
        }];
        // 同步配置
        [ECellCommandProvider syncConfig];
    }
    if (self.isJL)
    {
        if (IsDelegate(self.delegate, bluetoothPeripheral:didUpdateNotificationStateForCharacteristic:error:)) {
            [self.delegate bluetoothPeripheral:peripheral didUpdateNotificationStateForCharacteristic:characteristic error:error];
        }
    }
}

// 外设传来数据
- (void)peripheral:(CBPeripheral *)peripheral didUpdateValueForCharacteristic:(CBCharacteristic *)characteristic error:(NSError *)error {
    // 指令通道数据返回
    if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:kNotifyCharacteristic]]) {
        [self handleReceiveData:[characteristic value]];
    }
    if (IsDelegate(self.delegate, bluetoothPeripheral:didUpdateValueForCharacteristic:error:))
    {
        [self.delegate bluetoothPeripheral:peripheral didUpdateValueForCharacteristic:characteristic error:error];
    }
}

// 写入值的监听
- (void)peripheral:(CBPeripheral *)peripheral didWriteValueForCharacteristic:(CBCharacteristic *)characteristic error:(nullable NSError *)error {
    ECellLog(@"[BLUETOOTH DATA] WRITE SUCCESS");
}



#pragma mark - 工具方法

/**
 添加系统已配对的设备
 */
- (void)addPairedConnectedArray {
    NSArray *pairedDevices = [self.centralManager retrieveConnectedPeripheralsWithServices:@[[CBUUID UUIDWithString:kServiceUUID]]];
    for (CBPeripheral *peripheral in pairedDevices) {
        ECellPeripheralModel *peripheralModel = [[ECellPeripheralModel alloc] init];
        peripheralModel.peripheral = peripheral;
        peripheralModel.name = peripheral.name;
        peripheralModel.uuidStr = peripheral.identifier.UUIDString;
        peripheralModel.rssi = 0;
        [self.scanDevices addObject:peripheralModel];
        if (IsDelegate(self.delegate, bluetoothManager:allDevices:otaDevices:)) {
            [self.delegate bluetoothManager:self allDevices:self.scanDevices otaDevices:@[]];
        }
    }
}

/**
 获取设备绑定状态
 */
- (BOOL)isBind {
    ECellDeviceInfoModel *deviceInfoModel = [ECellDeviceInfoModel currentModel];
    NSLog(@"-->isBind");
    if (deviceInfoModel.macAddress) {
        return YES;
    }
    return NO;
}

/**
 根据序号查找当前队列是否存在
 */
- (ECellBLEItem *)requestItemFromIndex:(int)index {
    for (ECellBLEItem *item in self.commandBleItems) {
        if (item.sid == index) {
            return item;
        }
    }
    return nil;
}

/**
 检查data中某个字节中的某位的值 1:YES 0:NO
 
 @param data      数据包
 @param byteIndex 字节的下标
 @param bitIndex  位的下标(从左到又)
 */
- (BOOL)checkDataWithData:(NSMutableData *)data ByteIndex:(NSInteger)byteIndex bitIndex:(NSInteger)bitIndex {
    int num = ((Byte*)([data bytes]))[byteIndex];
    UInt8 confirm = (UInt8)num;
    confirm = confirm << bitIndex;
    confirm = confirm >> 7;
    return confirm;
}

/**
 CRC校验成功或失败后回复设备端
 */
- (void)checkCRCHandleWithData:(NSMutableData *)data confirm:(BOOL)confirm {
    if(data.length <=12) { //l1层就有13个字节了。不够长的数据是垃圾数据
        return;
    }
    Byte byte = confirm == YES ? [self stitchWithErr:1 Ack:1 Vision:0] : [self stitchWithErr:0 Ack:1 Vision:0];
    // 替换L1层第二个字节
    [data replaceBytesInRange:NSMakeRange(1, 1) withBytes:&byte length:1];
    // 替换长度为不带参数的5
    Byte buffer[2];
    buffer[0] = 5 >> 8;
    buffer[1] = 5 & 0xff;
    [data replaceBytesInRange:NSMakeRange(2, 2) withBytes:buffer length:2];
    Byte dataLength[2];
    dataLength[0] = 0;
    dataLength[1] = 0;
    [data replaceBytesInRange:NSMakeRange(11, 2) withBytes:dataLength length:2];
    // 发送L1层确认码给设备端
    [self.reconnectPeripheral writeValue:[data subdataWithRange:NSMakeRange(0, 13)] forCharacteristic:self.writeCharacteristic type:CBCharacteristicWriteWithoutResponse];
}

/**
 CRC校验数据是否正确
 */
- (BOOL)checkCRCWithData:(NSMutableData *)data {
    // 取出CRC
    Byte *bytes = (Byte *)[data bytes];
    // 利用位移管道运算转整数 modify by fangtao 2018.12.01
    int crc = (bytes[4] & 0xff) << 8 | (bytes[5] & 0xff);
    // L2层数据
    NSData *contentData = [data subdataWithRange:NSMakeRange(8, data.length - 8)];
    // 重新根据contentData计算CRC
    int dataCrc = 0xff;
    int contentLength = (int)contentData.length;
    Byte *contentBytes = (Byte *)[contentData bytes];
    for (int i = 0; i <contentLength ; i++) {
        dataCrc^= contentBytes[i] & 0xFF;
        for (int j = 0; j < 8; j++) {
            if((dataCrc & 1) !=0){
                dataCrc >>=1;
                dataCrc^=0xB8;
            }else{
                dataCrc>>=1;
            }
        }
    }
    // 如果相等则正确
    if (crc == dataCrc) {
        return YES;
    }
    return NO;
}

/**
 构造L1层第二个字节
 
 @param err    1:对的 0:错的
 @param ack    1:回复 0:命令
 @param vision 版本
 */
- (Byte)stitchWithErr:(int)err Ack:(int)ack Vision:(int)vision {
    int a = 0 * pow(2, 7) + 0 * pow(2, 6) + err * pow(2, 5) + ack * pow(2, 4) + vision;
    return a;
}

#pragma mark - 懒加载

- (NSMutableArray *)scanDevices {
    if (!_scanDevices) {
        _scanDevices = [[NSMutableArray alloc] init];
    }
    return _scanDevices;
}

- (NSMutableArray *)commandBleItems {
    if (!_commandBleItems) {
        _commandBleItems = [[NSMutableArray alloc] init];
    }
    return _commandBleItems;
}

void AfterDispatch(double delayInSeconds, dispatch_block_t _Nullable block) {
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(delayInSeconds * NSEC_PER_SEC)), dispatch_get_main_queue(), block);
}

@end
